==> getinstance-0.2-py3-none-any.whl: getinstance.py <==

from weakref import WeakSet


class InstanceManager:
    def __init__(self, owner=None, name=None):
        """
        owner should be provided if you want to add this manager dynamically 
        to the owner class.
        """
        if owner:
            self.__set_name__(owner, name)

    def get(self, **kwargs):
        for instance in getattr(self.owner, self.weakset):
            match = True
            for attr in kwargs:
                if not hasattr(instance, attr) or \
                   not getattr(instance, attr) == kwargs[attr]:
                    match = False
                    break
            if match:
                return instance

    def all(self):
        return getattr(self.owner, self.weakset)
    
    def filter(self, **kwargs):
        for instance in getattr(self.owner, self.weakset):
            if all(getattr(instance, x) == kwargs[x] for x in kwargs):
                yield instance

    def __set_name__(self, owner_class, name):
        """
        Called at the time the owning class `owner_class` is created. The InstanceManager 
        instance has been assigned to `name`.
        """
        assert owner_class and name
        self.weakset = f'_{name}_weakset'
        if hasattr(owner_class, self.weakset):
            return

        setattr(owner_class, self.weakset, WeakSet())
        self.owner = owner_class

        # Override owner class __new__ method so that each time
        # new instance is created, it is added to the `_instances`
        __new_original__ = getattr(owner_class, '__new__', None)
        
        def __new_wrapped__(cls, *args, **kwargs):
            if __new_original__ == object.__new__:
                instance = __new_original__(cls)
            else:
                instance = __new_original__(cls, *args, **kwargs)
            getattr(cls, self.weakset).add(instance)
            return instance
        
        owner_class.__new__ = __new_wrapped__

==> getinstance-0.3-py3-none-any.whl: getinstance.py <==

from weakref import WeakSet


class InstanceManager:
    def __init__(self, owner=None, name=None):
        """
        owner should be provided if you want to add this manager dynamically 
        to the owner class.
        """
        if owner:
            self.__set_name__(owner, name)

    def get(self, **kwargs):
        for instance in getattr(self.owner, self.weakset):
            match = True
            for attr in kwargs:
                if not hasattr(instance, attr) or \
                   not getattr(instance, attr) == kwargs[attr]:
                    match = False
                    break
            if match:
                return instance

    def all(self):
        return ProxyInstances(self)
    
    def filter(self, **kwargs):
        return ProxyInstances(self, kwargs)

    def __set_name__(self, owner_class, name):
        """
        Called at the time the owning class `owner_class` is created. The InstanceManager 
        instance has been assigned to `name`.
        """
        assert owner_class and name
        self.weakset = f'_{name}_weakset'
        if hasattr(owner_class, self.weakset):
            return

        setattr(owner_class, self.weakset, WeakSet())
        self.owner = owner_class

        # Override owner class __new__ method so that each time
        # new instance is created, it is added to the weakset
        __new_original__ = getattr(owner_class, '__new__', None)
        
        def __new_wrapped__(cls, *args, **kwargs):
            if __new_original__ == object.__new__:
                instance = __new_original__(cls)
            else:
                instance = __new_original__(cls, *args, **kwargs)
            getattr(cls, self.weakset).add(instance)
            return instance
        
        owner_class.__new__ = __new_wrapped__

 
class ProxyInstances:
    def __init__(self, manager, filter=None):
        self.manager = manager
        self.filter = filter
        
    def __iter__(self):
        if self.filter:
            for instance in getattr(self.manager.owner, self.manager.weakset):
                if all(getattr(instance, x) == self.filter[x] for x in self.filter):
                    yield instance
        else:
            for x in getattr(self.manager.owner, self.manager.weakset):
                yield x
        
    def __getattribute__(self, name):
        if name.startswith('_') or name in self.__dict__:
            return super().__getattribute__(name)
        return ProxyMethod(self, name)

class ProxyMethod:
    def __init__(self, instances, name):
        self.instances = instances
        self.name = name
        
    def __call__(self, *a, **kw):
        for instance in self.instances:
            getattr(instance, self.name)(*a, **kw)
